| package |
package := Package name: 'STT - Smalltalk Templates'.
package paxVersion: 0;
	basicComment: 'The same way ASP, JSP, PHP, etc. do it, STT''s idea is to make possible to write Smalltalk code within the context of an HTML, or any kind of document. For instance,

This is number {%= 5 + 2 %}, and this are repeating words:
{% 10 timesRepeat: [ %}
   ThisIsARepeatedWord
{% ] %}
is a simple template that STT can process to produce some output.

squeak author: Federico Stilman
port by: Diego Gomez Deck
see: http://minnow.cc.gatech.edu/squeak/2604
'.


package classNames
	add: #STTTemplate;
	yourself.

package binaryGlobalNames: (Set new
	yourself).

package globalAliases: (Set new
	yourself).

package allResourceNames: (Set new
	yourself).

package setPrerequisites: (IdentitySet new
	add: '..\Object Arts\Dolphin\Base\Dolphin';
	yourself).

package!

"Class Definitions"!

Object subclass: #STTTemplate
	instanceVariableNames: 'sttCode cache compiledTemplate asStringSelector'
	classVariableNames: ''
	poolDictionaries: ''
	classInstanceVariableNames: ''!

"Global Aliases"!


"Loose Methods"!

"End of package definition"!

"Source Globals"!

"Classes"!

STTTemplate guid: (GUID fromString: '{09B66270-A7F1-4947-96F5-043A3DE54680}')!
STTTemplate comment: 'I''m a Smalltalk Template. My escaping sequence for embedding Smalltalk expressions is: {% %} or {%= %}.

Try this expressions:

(STTTemplate on: ''Repeating word...{% 10 timesRepeat: [ %} WORD {% ] %}   end!!'') evaluateFor: nil.

(STTTemplate on: ''Counting: 
		{% self to: 10 do: [ :each | %}
			[{%= each %}]
		{% ] %}''
) evaluateFor: 1.
'!
!STTTemplate categoriesForClass!Unclassified! !
!STTTemplate methodsFor!

asSmalltalkCode
	"Returns the equivalent version of the receiver as Smalltalk code"

	| sttOpenIndex sttCloseIndex lastIndex sttCodeIndex stream smalltalkExpression |
	stream := String new writeStream.
	stream
		nextPutAll: 'doIt | out |';
		cr;
		nextPutAll: 'out := WriteStream on: String new.';
		cr.
	lastIndex := 1.
	[(sttOpenIndex := self sttCode indexOfSubCollection: '{%' startingAt: lastIndex) > 0] 
		whileTrue: 
			[self writeOutputCodeFor: (self sttCode copyFrom: lastIndex to: sttOpenIndex - 1)
				on: stream.
			sttCloseIndex := self sttCode 
						indexOfSubCollection: '%}'
						startingAt: sttOpenIndex
						ifAbsent: [^self error: 'Missing closing tag'].
			sttCodeIndex := sttOpenIndex + 2.
			(sttCode at: sttOpenIndex + 2) = $= 
				ifTrue: 
					[stream nextPutAll: 'out nextPutAll: ('.
					sttCodeIndex := sttCodeIndex + 1].
			smalltalkExpression := (sttCode copyFrom: sttCodeIndex to: sttCloseIndex - 1) trimBlanks.
			stream nextPutAll: smalltalkExpression.
			(sttCode at: sttOpenIndex + 2) = $= 
				ifTrue: 
					[stream nextPutAll: ') ' , self asStringSelector asString.
					sttCodeIndex := sttCodeIndex + 1].
			(',|[(.' includes: smalltalkExpression last) ifFalse: [stream nextPut: $.].
			stream cr.
			lastIndex := sttCloseIndex + 2].
	self writeOutputCodeFor: (self sttCode copyFrom: lastIndex to: sttCode size) on: stream.
	stream
		nextPutAll: '^ out contents';
		cr.
	^stream contents!

asStringSelector
	"Returns the selector used to show objects as Strings on the receiver"

	^asStringSelector!

asStringSelector: aSymbol
	"Sets the selector used to show objects as Strings on the receiver"

	asStringSelector := aSymbol!

cache
	"Returns the receiver's cached object"

	^cache!

cache: anObject
	"Save anObject in the receiver's cache"

	cache := anObject!

compileIn: aClass
	"Compiles the receiver's template in aClass and returns aCompiledMethod"

	| methodNode |
	methodNode := Compiler compile: self asSmalltalkCode in: aClass.
	^methodNode!

evaluateFor: anObject
	"Evaluates the receiver within for anObject"

	self isCached ifFalse: [self cache: (self compileIn: anObject class)].
	^self cache value: anObject withArguments: #()!

initializeCache
	"Initialize the receiver's cache"

	cache := nil!

initializeOn: aString asStringSelector: aSymbol
	sttCode := aString.
	asStringSelector := aSymbol.
	self initializeCache!

isCached
	"Tell if the receiver is cached or not. In the future, 
	this will consider the fact that a cached object may 
	become old after some time, and that means that the 
	object is NOT cached anymore."

	^self cache notNil!

sttCode
	"Returns the receiver's Smalltalk Template code"

	^sttCode!

writeOutputCodeFor: aString on: aStream
	"Writes on aStream the required Smalltalk code for outputing aString on 
	'out' "

	aStream
		nextPutAll: 'out nextPutAll: ''';
		nextPutAll: aString;
		nextPutAll: '''.';
		cr! !
!STTTemplate categoriesFor: #asSmalltalkCode!code generation!public! !
!STTTemplate categoriesFor: #asStringSelector!accessing!public! !
!STTTemplate categoriesFor: #asStringSelector:!accessing!public! !
!STTTemplate categoriesFor: #cache!caching!public! !
!STTTemplate categoriesFor: #cache:!caching!public! !
!STTTemplate categoriesFor: #compileIn:!compiling!public! !
!STTTemplate categoriesFor: #evaluateFor:!evaluating!public! !
!STTTemplate categoriesFor: #initializeCache!caching!private! !
!STTTemplate categoriesFor: #initializeOn:asStringSelector:!initializing!private! !
!STTTemplate categoriesFor: #isCached!caching!public! !
!STTTemplate categoriesFor: #sttCode!accessing!public! !
!STTTemplate categoriesFor: #writeOutputCodeFor:on:!code generation!public! !

!STTTemplate class methodsFor!

defaultAsStringSelector
	^#printString!

on: aString
	"Creates an instance of the receiver on aString"

	^self on: aString asStringSelector: self defaultAsStringSelector!

on: aString asStringSelector: aSymbol
	"Creates an instance of the receiver on aString"

	^self new initializeOn: aString asStringSelector: aSymbol!

test
	| sttTest |
	sttTest := '
		<HTML>
		<HEAD><TITLE>{%= self class %}</TITLE></HEAD>

		<BODY>
			<TABLE>
				{% self to: 10 do: [ :each | %}
			        <TR>
					   <TD>{%= each %}</TD>
					   <TD>{%= each * 2 %}</TD>
			        </TR>
			   	{% ] %}
			</TABLE>
		</BODY>
		</HTML>'.
	^(STTTemplate on: sttTest) evaluateFor: 1!

test2
	| sttTest |
	sttTest := '
		<HTML>
		<HEAD><TITLE>Another test</TITLE></HEAD>

		{% 
			out nextPutAll: ''This is a another test''.
			out nextPutAll: ''After all this ST code goes the final HTML closing tag''.
		
			15 timesRepeat: [ out nextPutAll: ''<P>This paragraph was manually send out</P>'' ]
		%}

		</HTML>'.
	^(STTTemplate on: sttTest) evaluateFor: 1! !
!STTTemplate class categoriesFor: #defaultAsStringSelector!defaults!public! !
!STTTemplate class categoriesFor: #on:!instance creation!public! !
!STTTemplate class categoriesFor: #on:asStringSelector:!instance creation!public! !
!STTTemplate class categoriesFor: #test!examples!public! !
!STTTemplate class categoriesFor: #test2!examples!public! !

"Binary Globals"!

"Resources"!

